{% extends "touchtechnology/news/archive/base.html" %}
{% load tz %}

{% block page_title %}{{ block.super }}: {{ year }}{% endblock %}

{% block archive_info %}
	<h2>{{ year }}</h2>

	<ul>
		{% for month in date_list %}
			<li><a href="{% timezone "UTC" %}{% url application.name|add:":month" year=month|date:"Y" month=month|date:"b" %}{% endtimezone %}">{{ month|date:"F Y" }}</a></li>
		{% endfor %}
	</ul>
{% endblock %}
