{% extends "touchtechnology/news/list_articles.html" %}
{% load i18n %}

{% block pagination %}
	<p class="buttons">
		<a class="button" href="{% url application.name|add:":archive" %}" title="Archive">{% trans "Visit the news archive." %}</a>
	</p>
{% endblock %}
