# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-02-03 06:19
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0023_competition__add__match_uuid_finalize'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='competition',
            options={'ordering': ('order',)},
        ),
        migrations.AlterModelOptions(
            name='division',
            options={'ordering': ('order',)},
        ),
        migrations.AlterModelOptions(
            name='season',
            options={'ordering': ('order',)},
        ),
    ]
