{% extends "competition/rankpoints_base.html" %}
{% load i18n %}

{% block content %}
	{% trans "Ranking" as title %}
	<h1>{{ node|default:title }}</h1>

	<h2>{{ object }}</h2>
	<h3>{{ day|date:"jS F Y" }}</h3>

	<div class="cms">
		{% for content in node.contents.all %}
			<div>
				{{ content.copy|safe }}
			</div>
		{% endfor %}
	</div>

	<table>
		{% for rank in object_list %}
			<tr>
				<td>{% ifchanged rank.points %}{{ forloop.counter }}{% else %}={% endifchanged %}</td>
				<td>{{ rank.team.club.title }}</td>
				<td><a href="{% url 'ranking:team' year=day.year month=day|date:"b" day=day|date:"j" slug=object.slug team=rank.team.club.slug %}">{{ rank.points }}</a></td>
			</tr>
		{% endfor %}
	</table>
{% endblock %}
